'''List of all endpoints we knows about.'''

ROOT = 'wss://atshop.io/websocket'
METHODS = {
    'order.exists': [
        'order_id'
    ],
    'order.fulfill.alternativePayment': [
        'order_id',
        'amount_paid',
        'payment_method'
    ],
    'admin.orders.replace': [
        'order_id',
        'shop_id',
        'quantitiy',
        'note'
    ],

    # Orders

    'admin.product.add': [
        'shop_id',
        'product'
    ],
    'admin.product.edit': [
        'product'
    ],
    'admin.product.remove': [
        'product_id'
    ],

    # Product

    'shop.product.groups.save': [
        'product_group'
    ],
    'shop.product.groups.remove': [
        'product_group_id'
    ],

    # Product Group

    'admin.product.stock.get.all': [
        'product_id'
    ],
    'admin.product.stock.add': [
        'product_id',
        'stock'
    ],
    'admin.product.stock.delete': [
        'stock_to_remove'
    ],
    'admin.product.stock.update': [
        'product_id',
        'to_add',
        'to_remove'
    ],

    # Stock

    'coupon.create': [
        'coupon'
    ],
    'coupon.edit': [
        'coupon'
    ],
    'coupon.remove': [
        'coupon_id'
    ]

    # Coupon Codes
}

# API_PATH  = {
#     'order'             : ROOT + '/orders',
#     'coupon'            : ROOT + '/coupons',
#     'products'          : ROOT + '/products',
#     'feedback'          : ROOT + '/feedback',
#     'blacklists'        : ROOT + '/blacklists',
#     'categories'        : ROOT + '/categories',
#
#     # Generic Endpoints
#
#     'queries'           : ROOT + '/queries',
#     'queries-reply'     : ROOT + '/queries/reply',
#     'queries-close'     : ROOT + '/queries/close',
#     'queries-reopen'    : ROOT + '/queries/reopen',
#
#     # Specific Query Endpoints
# }
# API_DATA = {
#     'categories': {
#         'title': 1,
#         'unlisted': 0,
#         'sort_priority': 0,
#         'products_bound': 0
#     },
#     'blacklists': {
#         'type': 1,
#         'data': 1,
#         'note': 0
#     },
#     'feedback': {
#         'reply': 1
#     },
#     'products': {
#         'title': 1,
#         'description': 1,
#         'price': 1,
#         'gateways': 1,
#         'type': 1,
#         'discount_value': 1,
#         'currency': 0,
#         'quantity': 0,
#         'stock_delimiter': 0,
#         'serials': 0,
#         'serials_remove_duplicates': 0,
#         'delivery_text': 0,
#         'service_text': 0,
#         'stock': 0,
#         'custom_fields': 0,
#         'crypto_confirmations_needed': 0,
#         'max_risk_leve': 0,
#         'block_vpn_proxies': 0,
#         'sort_priority': 0,
#         'unlisted': 0,
#         'terms_of_service': 0,
#         'warranty': 0,
#         'warranty_text': 0,
#         'private': 0,
#         'webhooks': 0
#     },
#     'coupon': {
#         'code': 1,
#         'discount_value': 1,
#         'max_uses': 0,
#         'products_bound': 0
#     },
#     'queries-reply': {
#         'reply': 1
#     }
# }



import json
import websocket
from threading import Thread, Event

from .endpoints import *

class Opcode:
    PING            = 'ping'
    RESULT          = 'result'
    CONNECTED       = 'connected'

class Server():
    '''
    This class is our middleman between us and atshop,
    and it handles everything related to websockets.
    '''

    def __init__(self):
        self._id = 0
        self._login = False

        self._connected = Event()
        self._response = Event()
        self._response_data = None

        self._websocket = self.create_websocket()

        self.run()

    def create_websocket(self):
        return websocket.WebSocketApp(
            url=ROOT,
            on_open=lambda ws: self.on_open(ws),
            on_message=lambda ws, msg: self.on_message(ws, msg),
            on_error=lambda ws, msg: self.on_error(ws, msg),
            on_close=lambda ws, close_code, close_msg: self.on_close(ws, close_code, close_msg)
        )

    def on_open(self, ws):
        self.send(
            {
                'msg': 'connect',
                'version': '1',
                'support': [
                    '1',
                    'pre2',
                    'pre1'
                ]
            }
        )

    def on_close(self, ws, close_code, close_msg):
        self.connected = False

    def on_error(self, ws, error):
        raise Exception(error)

    def on_message(self, ws, message):
        data = json.loads(message)
        op = data.get('msg')

        if op == Opcode.RESULT and self._login:
            if data.get('error'):
                raise Exception(
                    data['error']['reason']
                )

            else:
                self._login = False

        elif op == Opcode.PING:
            self.send(
                {
                    'msg': 'pong'
                }
            )
            # Heartbeat

        elif op == Opcode.CONNECTED:
            self._connected.set()

        elif op == Opcode.RESULT and self._id >= 11:
            self._response_data = data
            self._response.set()

        self._id += 1

    def send(self, payload):
        self._websocket.send(
            json.dumps(payload)
        )

        self._id += 1

    def run(self):
        Thread(
            target=self._websocket.run_forever,
            kwargs={
                'ping_interval': 10,
                'ping_timeout': 5
            },
            # daemon=True
        ).start()

        self._connected.wait()

        # Waiting for the websocket to connect to atshop
        # if we do not include this, our code will begin
        # executing before we want to. The result of this is
        # websocket._exceptions.WebSocketConnectionClosedException

class Client(Server):
    '''
    This will be how users interact
    with our server class, mainly through
    the use of our "request" function.
    '''

    def login(self, token):
        self.send(
            {
                'msg': 'method',
                'id': str(self._id),
                'method': 'login',
                'params': [
                    {
                        'resume': token
                    }
                ]
            }
        )

        self._id += 1
        self._login = True

    def request(self, **kwargs):
        method = kwargs['method']
        method_args = METHODS.get(method)

        if all(
            arg in kwargs
            for arg in method_args
        ):
            del kwargs['method']

            self.send(
                {
                    'msg': 'method',
                    'id': str(self._id),
                    'method': method,
                    'params': [
                        *kwargs.values()
                    ]
                }
            )

            self._response.wait()
            self._response.clear()

            return self._response_data



from atshop import Client

client = Client()
client.login('token')

product = client.request(
    method='admin.product.add',

    shop_id='4AueTN7mx6xRurrfu',
    product={
        'name': 'Clap is cool <3',
        'description': '',
        'purchaseNotes': '',
        'value': 100,
        'icon': '',
        'minQuantity': 1,
        'maxQuantity': 0,
        'image_url': '',
        'style': 'box',
        'requireShipping': 'no',
        'paymentMethods': [],
        'maxDisplayedStock': 0,
        'preventDuplicates': True,
        'priority': 0,
        'category': '',
        'displayDescription': True,
        'useOrderIdAsItemName': False,
        'hidden': False,
        'notForSale': False
    }
)

print(product['result'])



from setuptools import find_packages, setup

PACKAGE_NAME = 'atshop'
VERSION = '1.0.0'

with open('README.md', 'r') as readme:
    long_description = readme.read()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author='iclapcheeks',
    python_requires='~=3.7',
    author_email='iclapcheeeeeks@protonmail.com',
    description='A Python wrapper for the Atshop API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/iclapcheeks/Atshop',
    packages=find_packages(),
    install_requires=[
        'websocket_client>=1.3.3'
    ]
)
